[license-shield-url]: https://img.shields.io/github/license/typeforce-js/immutable-dts.svg?style=flat-square
[license-url]: https://github.com/Typeforce-JS/immutable-dts/blob/master/LICENSE
[npm-downloads-shield-url]: https://img.shields.io/npm/dt/@typeforce/immutable-dts.svg?style=flat-square
[npm-version-shield-url]: https://img.shields.io/npm/v/@typeforce/immutable-dts.svg?style=flat-square
[npm-url]: https://www.npmjs.com/package/@typeforce/immutable-dts

# immutable-dts

[![NPM][npm-version-shield-url]][npm-url]
[![NPM][npm-downloads-shield-url]][npm-url]
[![LICENSE][license-shield-url]][license-url]

Typescript type definitions for treating objects as immutable.

## Usage

```shell
    npm install @typeforce/immutable-dts --save   # Install package via NPM
```

```typescript
    import { Immutable } from "@typeforce/immutable-dts";

    type Account = {
        name: string,
        age: number,
        obj: { x: number, y: number };
    };

    // An easy way of making an object's properties read-only is using the
    // Typescript `Readonly` type.
    const john: Readonly<Account> = {
        name: "John Doe",
        age: 42,
        obj: {
            x: 5,
            y: 3
        }
    };

    // However, `Readonly` only protects properties that are immediate children
    // of their object; it is not recursive.
    john.obj.x = 6; // <- This should not be possible! We made `john` readonly!

    // `Immutable` recursively declares object properties as read-only, so the
    // object cannot be modified.
    const immutableJohn = john as Immutable<Account>;
    account.obj.x = 6; // <- Compiler error: `x` is readonly!
```

This package contains TypeScript type declarations.

## Build

When building the project, a folder named `dist/` will be created if it does not
already exist, where the compiled code will be outputted to. Type declaration
files will also be generated and outputted to the `dist/` folder.

```shell
    npm install     # Installs dependencies.
    npm run build   # Build the project.
```

## License

Refer to the `LICENSE` file for license information.
